--
-- Sneekee Mods - Simple UI and Zone Map Changes
-- Author: Polgara97
--

local DEFAULTS = {
    profile = {
        autoshow = true,
	bgonly = false,
	hideedges = true,
	hideclose = true,
	fpsmon = true,
	fpsmonon = true,
	hidegryph = true,
        mapsize = 1.0,
        hidemacro = false,
        hidekeybinds = false,
        hidequests = false,
        movequests = true,
        fpscolor = 7,
	acamignore = true,
	acamon = false,
	acambasic = false,
        usemaptools = true,
        garreport = true,
        hideminimapicon = false,
        classcolorsoff = false,
        acamminimap = false,
    },
    char = {
        wmapsize = 1.0,
        movemapx = -900,
        movemapx = 100,
        snkepoint = "CENTER",
        snkerelto = nil,
        snkerelpt = "CENTER",
        snkemapdrag = true,
    },
}

--------------------------------------------------------------------------------
-- Initialization
--------------------------------------------------------------------------------
SneekeeMods = LibStub("AceAddon-3.0"):NewAddon("SneekeeMods", "AceConsole-3.0", "AceEvent-3.0")

-- Minimap Button
local icon = LibStub("LibDBIcon-1.0")

local loadAttempt = 1; -- Fix Ace/Blizz issue with showing the wrong screen on first call

function SneekeeMods:OnInitialize()
    -- User's settings
    self.db = LibStub("AceDB-3.0"):New("SneekeeModsDB", DEFAULTS, true)

    LibStub("AceConfig-3.0"):RegisterOptionsTable("SneekeeMods", SneekeeModsConfig.AceConfig)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SneekeeMods", "SneekeeMods")

    -- Minimap Button
    local sneekeeLDB = LibStub("LibDataBroker-1.1"):NewDataObject("SneekeeCFG", {
        type = "data source",
        text = "SneekeeConfig",
        icon = "Interface\\AddOns\\SneekeeMods\\Images\\Sneekee",
        OnClick = function(_,button) 
            if IsShiftKeyDown() then
                if self.db.profile.acamminimap == true then
                    if self.db.profile.acamignore == true then
                        self:Print("|cffff0000Please Allow SneekeeMods to Control Your ActionCam in Settings");
                    else
                        if button == "RightButton" then
                            SneekeeMods.db.profile.acamon = false;
                            ConsoleExec( "ActionCam off" );
                        else
                            SneekeeMods.db.profile.acamon = true;
                            if self.db.profile.acambasic == true then
                                ConsoleExec( "ActionCam basic" );
                            else
                                ConsoleExec( "ActionCam full" );
                            end
                        end
                    end
                end
            elseif IsControlKeyDown() then
                if BattlefieldMapFrame:IsVisible() then
                    BattlefieldMapFrame:Hide();
                    self.db.profile.autoshow = false;
                    self:Print("Zone Map Auto-Show is Now Off (|cffff0000reload required|r)");
                else
                    BattlefieldMapFrame:Show();
                    self.db.profile.autoshow = true;
                    self:Print("Zone Map Auto-Show is Now On (|cffff0000reload required|r)");
                end
            else
                if button == "RightButton" then
                    SneekeeMods:FullScreenToggle();
                else
                    SneekeeMods:SneekeeOptions();
                end
            end
        end,
    })

    icon:Register("SneekeeCFG", sneekeeLDB, SneekeeMods.db.profile.snkeminimap)
--    icon:Register("SneekeeCFG", sneekeeLDB, false)
    function sneekeeLDB.OnTooltipShow(tt)
        tt:AddLine("|CFF00CCFFSneekeeMods")
        tt:AddLine("|CFFE6CC7FClick|r for Settings")
        tt:AddLine("|CFFE6CC7FRight-Click|r to Toggle Fullscreen")
        tt:AddLine("|CFFE6CC7FCtrl-Click|r to Toggle Zone Map")
        if self.db.profile.acamminimap == true then
            tt:AddLine("|CFFE6CC7FShift-Left-Click:|r ActionCam On")
            tt:AddLine("|CFFE6CC7FShift-Right-Click:|r ActionCam Off")
        end
    end

    self:RegisterChatCommand("snke", "ChatCommand")
    self:RegisterChatCommand("sneekee", "ChatCommand")
end

--------------------------------------------------------------------------------
-- Toggle Minimap Icon
--------------------------------------------------------------------------------
function SneekeeMods.CheckMMicon()
    if SneekeeMods.db.profile.hideminimapicon then
	icon:Hide("SneekeeCFG")
    else
	icon:Show("SneekeeCFG")
    end
end

--------------------------------------------------------------------------------
-- Slash Commands
--------------------------------------------------------------------------------
function SneekeeMods:ChatCommand(input)
    if not input or input:trim() == "" then
        SneekeeMods:SneekeeOptions();
  elseif input:trim() == "fs" then
     SneekeeMods:FullScreenToggle();
    else
        LibStub("AceConfigCmd-3.0"):HandleCommand("snke", "SneekeeMods", input);
        self:Print("Don't be sneekee!");
    end
end

--------------------------------------------------------------------------------
-- Register events & commands
--------------------------------------------------------------------------------
--SneekeeMods:RegisterEvent("PLAYER_ENTERING_WORLD")

--------------------------------------------------------------------------------
-- FPS Monitor
--------------------------------------------------------------------------------
local timer = 0;
local FPS_Tracking_Frame = CreateFrame ( "Frame" , "FPS_Tracking_Frame" , UIParent );
local FPS_Tracking_FrameText = FPS_Tracking_Frame:CreateFontString ( "FPS_Tracking_FrameText" , "OVERLAY" , "GameFontNormal" );
-- Core Frame
--FPS_Tracking_Frame:ClearAllPoints(); --RESET
--FPS_Tracking_Frame:SetPoint("CENTER" , UIPanel); --RESET
--FPS_Tracking_Frame:SetPoint(0,0); --RESET
FPS_Tracking_Frame:SetPoint ( "CENTER" , UIParent );
FPS_Tracking_Frame:SetSize ( 75 , 15 );
FPS_Tracking_Frame:EnableMouse ( true );
FPS_Tracking_Frame:SetMovable ( true );
FPS_Tracking_Frame:SetUserPlaced ( true );
FPS_Tracking_Frame:SetToplevel ( true );
FPS_Tracking_Frame:RegisterForDrag ( "LeftButton" );
FPS_Tracking_Frame:SetScript ( "OnDragStart" , FPS_Tracking_Frame.StartMoving );
FPS_Tracking_Frame:SetScript ( "OnDragStop" , FPS_Tracking_Frame.StopMovingOrSizing );
-- FPS Frame Text
FPS_Tracking_FrameText:SetPoint ( "LEFT" , FPS_Tracking_Frame );
FPS_Tracking_FrameText:SetFont ( STANDARD_TEXT_FONT , 16 );
FPS_Tracking_FrameText:Hide();
    
FPS_Tracking_Frame:SetScript ( "OnUpdate" , function ( self , elapsed )
    timer = timer + elapsed;
    if timer > 0.05 then
        if FramerateLabel:IsVisible() then
            FPS_Tracking_FrameText:Show();
            if (FramerateText:GetText() ~= nil) then
                FPS_Tracking_FrameText:SetText ( "FPS: " .. FramerateText:GetText() );
            end
        else
            FPS_Tracking_FrameText:Hide();
        end
        timer = 0;
    end
end);

function SneekeeMods:CheckFPSMonitor()
    if self.db.profile.fpsmon then
        -- Going to make the other frame invisible
        FramerateLabel:SetTextColor ( 1 , 1 , 1 , 0 );
        FramerateText:SetTextColor ( 1 , 1 , 1 , 0 );
        FPS_Tracking_Frame:Show();
    else
        -- Don't show the new monitor
        FPS_Tracking_Frame:Hide();
        FramerateLabel:SetTextColor ( 1 , 1 , 1 );
        FramerateText:SetTextColor ( 1 , 1 , 1 );
    end
end

function SneekeeMods:CheckFPSColor()
    if self.db.profile.fpscolor == 1 then -- White
        FPS_Tracking_FrameText:SetTextColor ( 1 , 1 , 1 );
    elseif self.db.profile.fpscolor == 2 then -- Yellow
        FPS_Tracking_FrameText:SetTextColor ( 1 , 0.96 , 0.41 );
    elseif self.db.profile.fpscolor == 3 then -- Red
        FPS_Tracking_FrameText:SetTextColor ( 0.77 , 0.12 , 0.23 );
    elseif self.db.profile.fpscolor == 4 then -- Green
        FPS_Tracking_FrameText:SetTextColor ( 0.67 , 0.83 , 0.45 );
    elseif self.db.profile.fpscolor == 5 then -- Blue
        FPS_Tracking_FrameText:SetTextColor ( 0.0 , 0.44 , 0.87 );
    elseif self.db.profile.fpscolor == 6 then -- Purple
        FPS_Tracking_FrameText:SetTextColor ( 0.64, 0.19 , 0.79 );
    else -- Grey
        FPS_Tracking_FrameText:SetTextColor ( 0.5 , 0.5 , 0.5 , 1 );
    end

end

--------------------------------------------------------------------------------
-- Main
--------------------------------------------------------------------------------
function SneekeeMods:OnEnable()
    SneekeeMods:CheckGryphons();

    -- Zone Minimap (shift-m map)
    -- local scale = 0.88
    local scale = self.db.profile.mapsize
    local f = CreateFrame("Frame")
    local SetMiniMap = function()
        BattlefieldMapFrame:SetScale(scale);
        SneekeeMods:CheckMapEdges();
        SneekeeMods:CheckCloseButton();
        BattlefieldMapFrame:Hide();

        if self.db.profile.bgonly == false then
            BattlefieldMapFrame:Show();
        elseif self.db.profile.bgonly == true then
            if UnitInBattleground("player") then
                BattlefieldMapFrame:Show();
            else
                BattlefieldMapFrame:Hide();
            end
        end

    end
    f:SetScript("OnEvent",function()
        -- Modify Action Bar Buttons
        for i = 1, 12 do
            SneekeeMods:FixButton(_G["MultiBarBottomLeftButton"..i])
            SneekeeMods:FixButton(_G["MultiBarBottomRightButton"..i])
            SneekeeMods:FixButton(_G["ActionButton"..i])
            SneekeeMods:FixButton(_G["MultiBarRightButton"..i])
            SneekeeMods:FixButton(_G["MultiBarLeftButton"..i])
            SneekeeMods:FixButton(_G["ButtonForge"..i])
        end
        if not BattlefieldMapFrame then
            LoadAddOn("Blizzard_BattlefieldMapFrame")BattlefieldMap_LoadUI();
        end
        if self.db.profile.autoshow == true then
            SetMiniMap(); -- Update zone map if it is enabled
        end
            SneekeeMods.CheckMMicon(); -- Set the SneekeeMods minimap icon
    end)
    f:RegisterEvent("ZONE_CHANGED")
    f:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    f:RegisterEvent("PLAYER_ENTERING_WORLD")

    if self.db.profile.hidequests then
        SneekeeMods:CollapseQuests();
    end
    if self.db.profile.movequests then
        SneekeeMods:QuestMover();
    end

    SneekeeMods:CheckFPSMonitor();

    SneekeeMods:CheckFPSColor();

    if self.db.profile.fpsmonon then
        ToggleFramerate();
    end

    SneekeeMods:CheckActionCam();

    if SneekeeMods.db.profile.garreport then
        SneekeeMods:AddGarrisonButton()
    end
end

--------------------------------------------------------------------------------
-- Zone Map Size
--------------------------------------------------------------------------------
function SneekeeMods:ChangeMapSize()
    BattlefieldMapFrame:SetScale(self.db.profile.mapsize);
end

--------------------------------------------------------------------------------
-- Zone Map Edges
--------------------------------------------------------------------------------
function SneekeeMods:CheckMapEdges()
    if self.db.profile.hideedges then
        SneekeeMods:HideMapEdges();
    else
        SneekeeMods:ShowMapEdges();
    end
end
function SneekeeMods:HideMapEdges()
    BattlefieldMapFrame.BorderFrame.Top:Hide();
    BattlefieldMapFrame.BorderFrame.TopLeft:Hide();
    BattlefieldMapFrame.BorderFrame.TopRight:Hide();
    BattlefieldMapFrame.BorderFrame.Left:Hide();
    BattlefieldMapFrame.BorderFrame.BottomLeft:Hide();
    BattlefieldMapFrame.BorderFrame.Right:Hide();
    BattlefieldMapFrame.BorderFrame.BottomRight:Hide();
    BattlefieldMapFrame.BorderFrame.Bottom:Hide();
    BattlefieldMapFrame.BorderFrame.CloseButtonBorder:Hide();
end
function SneekeeMods:ShowMapEdges()
    BattlefieldMapFrame.BorderFrame.Top:Show();
    BattlefieldMapFrame.BorderFrame.TopLeft:Show();
    BattlefieldMapFrame.BorderFrame.TopRight:Show();
    BattlefieldMapFrame.BorderFrame.Left:Show();
    BattlefieldMapFrame.BorderFrame.BottomLeft:Show();
    BattlefieldMapFrame.BorderFrame.Right:Show();
    BattlefieldMapFrame.BorderFrame.BottomRight:Show();
    BattlefieldMapFrame.BorderFrame.Bottom:Show();
    if self.db.profile.hideclose then
        BattlefieldMapFrame.BorderFrame.CloseButtonBorder:Hide();
    else
        BattlefieldMapFrame.BorderFrame.CloseButtonBorder:Show();
    end
end

--------------------------------------------------------------------------------
-- Zone Map Close Button
--------------------------------------------------------------------------------
function SneekeeMods:CheckCloseButton()
    if self.db.profile.hideclose then
        SneekeeMods:HideCloseButton();
    else
        SneekeeMods:ShowCloseButton();
    end
end
function SneekeeMods:HideCloseButton()
    BattlefieldMapFrame.BorderFrame.CloseButton:Hide();
    BattlefieldMapFrame.BorderFrame.CloseButtonBorder:Hide();
end
function SneekeeMods:ShowCloseButton()
    BattlefieldMapFrame.BorderFrame.CloseButton:Show();
    if self.db.profile.hideedges then
        BattlefieldMapFrame.BorderFrame.CloseButtonBorder:Hide();
    end
end

--------------------------------------------------------------------------------
-- Action Cam
--------------------------------------------------------------------------------
function SneekeeMods:CheckActionCam()
    if self.db.profile.acamon == true and self.db.profile.acambasic == false and self.db.profile.acamignore == false then
        ConsoleExec( "ActionCam full" );
    elseif self.db.profile.acamon == true and self.db.profile.acambasic == true and self.db.profile.acamignore == false then
        ConsoleExec( "ActionCam basic" );
    elseif self.db.profile.acamon == false and self.db.profile.acamignore == false then
        ConsoleExec( "ActionCam off" );
    end
end

--------------------------------------------------------------------------------
-- Action Bar Gryphons
--------------------------------------------------------------------------------
function SneekeeMods:CheckGryphons()
    if self.db.profile.hidegryph then
        MainMenuBarArtFrame.LeftEndCap:Hide();
        MainMenuBarArtFrame.RightEndCap:Hide();
    else
        MainMenuBarArtFrame.LeftEndCap:Show();
        MainMenuBarArtFrame.RightEndCap:Show();
    end
end

--------------------------------------------------------------------------------
-- World Map Mover
--------------------------------------------------------------------------------
WorldMapFrame.isMoving = false;
WorldMapFrame:SetMovable ( true );
WorldMapFrame:EnableMouse ( true );
WorldMapFrame:SetUserPlaced ( true );
WorldMapFrame:RegisterForDrag ( "LeftButton" );
WorldMapFrame:SetScript("OnMouseDown", function(self, button)
    if button == "LeftButton" and not WorldMapFrame.isMoving and SneekeeMods.db.profile.usemaptools and SneekeeMods.db.char.snkemapdrag and not WorldMapFrame:IsMaximized() then
        WorldMapFrame:StartMoving();
        WorldMapFrame.isMoving = true;
    end
end)
WorldMapFrame:SetScript("OnMouseUp", function(self, button)
    if button == "LeftButton" and WorldMapFrame.isMoving then
        WorldMapFrame:StopMovingOrSizing();
        SneekeeMods.SetMapInfo();
        WorldMapFrame.isMoving = false;
    end
end)

function SneekeeMods.SetMapInfo()
    point, relativeTo, relativePoint, xOfs, yOfs = WorldMapFrame:GetPoint();
    SnkMapx = (math.floor(xOfs * (10^2)+ 0.5 ) / 10^2);
    SnkMapy = (math.floor(yOfs * (10^2)+ 0.5 ) / 10^2);
    SneekeeMods.db.char.movemapx = SnkMapx;
    SneekeeMods.db.char.movemapy = SnkMapy;
    SneekeeMods.db.char.snkepoint = point;
    SneekeeMods.db.char.snkerelto = relativeTo;
    SneekeeMods.db.char.snkerelpt = relativePoint;
end

function SneekeeMods:SneekeeMapOn()
    WorldMapFrame:ClearAllPoints();
    SneekeeMods:CheckWorldMax();
    WorldMapFrame:SetScale(SneekeeMods.db.char.wmapsize);
end
function SneekeeMods:SneekeeMapOff()
    SneekeeMods:CheckWorldMax();
    WorldMapFrame:Hide();
end
function SneekeeMods:CheckWorldMap()
    WorldMapFrame:Show();
    if SneekeeMods.db.profile.usemaptools then
        SneekeeMods:SneekeeMapOn();
    else
        SneekeeMods:SneekeeMapOff();
    end
end
function SneekeeMods:CheckWorldMax()
    if WorldMapFrame:IsMaximized() then
        WorldMapFrame:SetPoint("CENTER",UIParent,"CENTER", 0, 0);
    elseif SneekeeMods.db.profile.usemaptools then
        WorldMapFrame:SetPoint(SneekeeMods.db.char.snkepoint, SneekeeMods.db.char.snkerelto, SneekeeMods.db.char.snkerelpt, SneekeeMods.db.char.movemapx, SneekeeMods.db.char.movemapy);
    end
end

local m=CreateFrame("Frame",nil,WorldMapFrame)
local timer = 0
m:SetScript("OnUpdate",function(self,elapsed)
    timer = timer + elapsed
    if timer > 0.005 then
        timer = 0
        if SneekeeMods.db.profile.usemaptools and WorldMapFrame.isMoving == false then
            SneekeeMods:SneekeeMapOn();
        elseif SneekeeMods.db.profile.usemaptools == false then
            WorldMapFrame:SetScale(1.0);
        end
    end
end)

--------------------------------------------------------------------------------
-- Add Garrison Report to Order Hall Button
--------------------------------------------------------------------------------
function SneekeeMods:AddGarrisonButton()
    GarrisonLandingPageMinimapButton:HookScript("OnEnter",function(this)
        if this.description==GARRISON_TYPE_8_0_LANDING_PAGE_TOOLTIP then
            GameTooltip:AddLine("Right Click to Open Class Hall Report ")
            GameTooltip:AddLine("Shift-Click to Open Garrison Report ")
        end
        GameTooltip:Show()
    end)
    GarrisonLandingPageMinimapButton:RegisterForClicks("LEFTBUTTONUP","RIGHTBUTTONUP")
    GarrisonLandingPageMinimapButton:SetScript("OnClick",
        function (this,button)
            if (_G.GarrisonLandingPage and GarrisonLandingPage:IsShown()) then
                HideUIPanel(GarrisonLandingPage);
            else
                if IsShiftKeyDown() then
                    ShowGarrisonLandingPage(2)
                elseif button=="RightButton" then
                    ShowGarrisonLandingPage(3)
                else
                    ShowGarrisonLandingPage()
                end
            end
    end)
end

--------------------------------------------------------------------------------
-- Chat Class Colors
--------------------------------------------------------------------------------
function SneekeeMods:CheckClassColor()
    if self.db.profile.classcolorsoff then
        ConsoleExec( "SET chatClassColorOverride 1" );
    else
        ConsoleExec( "SET chatClassColorOverride 0" );
    end
end

--------------------------------------------------------------------------------
-- Full Screen Toggle
--------------------------------------------------------------------------------
function SneekeeMods:FullScreenToggle()
    local FSvalue = GetCVar("gxMaximize");
    if FSvalue == "1" then
        SetCVar("gxMaximize", "0")
    else
        SetCVar("gxMaximize", "1")
    end
    RestartGx();
end

--------------------------------------------------------------------------------
-- Open Options
--------------------------------------------------------------------------------
function SneekeeMods:SneekeeOptions()
    if loadAttempt == 1 then -- Fix Ace/Blizz issue with showing the wrong screen on first call
        loadAttempt = loadAttempt + 1;
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
    end
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
end

--------------------------------------------------------------------------------
-- Shift-Click to Move Quest Tracker
--------------------------------------------------------------------------------
--ObjectiveTrackerFrame:ClearAllPoints(); --RESET
--ObjectiveTrackerFrame:SetPoint("CENTER" , UIPanel); --RESET
--ObjectiveTrackerFrame:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOM", -5, -5) 
----ObjectiveTrackerFrame:SetPoint(0,0); --RESET
----/run ObjectiveTrackerFrame:SetHeight(650)

    local QT = ObjectiveTrackerFrame
    QT.ClearAllPoints = function() end
    QT:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOM", -45, -5) 
    QT.SetPoint = function() end
function SneekeeMods:QuestMover()
    QT:SetMovable(true)
    QT:SetUserPlaced(true)
    QT:SetClampedToScreen(true)
    QT:SetHeight(550)
    QT:SetWidth(190)

    local MoveQuestTracker = CreateFrame("Frame", nil, QT)  
    MoveQuestTracker:SetHeight(15)
    MoveQuestTracker:ClearAllPoints()
    MoveQuestTracker:SetPoint("TOPLEFT", QT)
    MoveQuestTracker:SetPoint("TOPRIGHT", QT)
    MoveQuestTracker:EnableMouse(true)
    MoveQuestTracker:SetHitRectInsets(-5, -5, -5, -5)
    MoveQuestTracker:RegisterForDrag("LeftButton")
    MoveQuestTracker:SetScript("OnDragStart", function(self, button)
        if button=="LeftButton" and IsModifiedClick()then
            QT:StartMoving()
        end
    end)
    MoveQuestTracker:SetScript("OnDragStop", function(self, button)
        QT:StopMovingOrSizing()
    end)
end
--------------------------------------------------------------------------------
-- Collapse Quests Frame & Trigger Button Cleanup
--------------------------------------------------------------------------------
local a = CreateFrame("Frame")
a:RegisterEvent("PLAYER_LOGIN")
a:RegisterEvent("PLAYER_ENTERING_WORLD")
a:RegisterEvent("ADDON_LOADED")

function SneekeeMods:CollapseQuests()
    -- Auto Minimize Quest Objectives at Login and set the minimized Objectives text and button to 50% transparent.
    a:SetScript("OnEvent",function(self,event)
        if event == "PLAYER_LOGIN" then
            ObjectiveTracker_Collapse()
            --	ObjectiveTrackerFrame.HeaderMenu:SetAlpha(.50)
        end
    end)
end
--------------------------------------------------------------------------------
-- Modify Action Bar Buttons
--------------------------------------------------------------------------------
function SneekeeMods:FixButton(button)
    if self.db.profile.hidekeybinds then
        button.HotKey:SetAlpha(0)  -- Hide keybinds on buttons
    end
    if self.db.profile.hidemacro then
        if button then
            button.Name:Hide()         -- Hide macro names on buttons
        end
    end    
end

